Changelog — Alembic 0.7.0 documentation

### Navigation

- index
- modules |
- previous |
- Alembic 0.7.0 documentation »

# Changelog

## 0.7.0

Released: November 24, 2014

### changed

- [changed] [commands] The `--head_only` option to the `alembic current` command is deprecated; the `current` command now lists just the version numbers alone by default; use `--verbose` to get at additional output.

- [changed] [autogenerate] The default value of the `EnvironmentContext.configure.user_module_prefix` parameter is no longer the same as the SQLAlchemy prefix. When omitted, user-defined types will now use the `__module__` attribute of the type class itself when rendering in an autogenerated module.

  References: #229

- [changed] [compatibility] Minimum SQLAlchemy version is now 0.7.6, however at least 0.8.4 is strongly recommended. The overhaul of the test suite allows for fully passing tests on all SQLAlchemy versions from 0.7.6 on forward.

### feature

- [feature] [versioning] The “multiple heads / branches” feature has now landed. This is by far the most significant change Alembic has seen since its inception; while the workflow of most commands hasn’t changed, and the format of version files and the `alembic_version` table are unchanged as well, a new suite of features opens up in the case where multiple version files refer to the same parent, or to the “base”. Merging of branches, operating across distinct named heads, and multiple independent bases are now all supported. The feature incurs radical changes to the internals of versioning and traversal, and should be treated as “beta mode” for the next several subsequent releases within 0.7.

  See also

  Working with Branches

  References: #167

- [feature] [versioning] In conjunction with support for multiple independent bases, the specific version directories are now also configurable to include multiple, user-defined directories. When multiple directories exist, the creation of a revision file with no down revision requires that the starting directory is indicated; the creation of subsequent revisions along that lineage will then automatically use that directory for new files.

  See also

  Setting up Multiple Version Directories

  References: #124

- [feature] [operations] [sqlite] Added “move and copy” workflow, where a table to be altered is copied to a new one with the new structure and the old one dropped, is now implemented for SQLite as well as all database backends in general using the new `Operations.batch_alter_table()` system. This directive provides a table-specific operations context which gathers column- and constraint-level mutations specific to that table, and at the end of the context creates a new table combining the structure of the old one with the given changes, copies data from old table to new, and finally drops the old table, renaming the new one to the existing name. This is required for fully featured SQLite migrations, as SQLite has very little support for the traditional ALTER directive. The batch directive is intended to produce code that is still compatible with other databases, in that the “move and copy” process only occurs for SQLite by default, while still providing some level of sanity to SQLite’s requirement by allowing multiple table mutation operations to proceed within one “move and copy” as well as providing explicit control over when this operation actually occurs. The “move and copy” feature may be optionally applied to other backends as well, however dealing with referential integrity constraints from other tables must still be handled explicitly.

  See also

  Running “Batch” Migrations for SQLite and Other Databases

  References: #21

- [feature] [commands] Relative revision identifiers as used with `alembic upgrade`, `alembic downgrade` and `alembic history` can be combined with specific revisions as well, e.g. `alembic upgrade ae10+3`, to produce a migration target relative to the given exact version.

- [feature] [commands] New commands added: `alembic show`, `alembic heads` and `alembic merge`. Also, a new option `--verbose` has been added to several informational commands, such as `alembic history`, `alembic current`, `alembic branches`, and `alembic heads`. `alembic revision` also contains several new options used within the new branch management system. The output of commands has been altered in many cases to support new fields and attributes; the `history` command in particular now returns it’s “verbose” output only if `--verbose` is sent; without this flag it reverts to it’s older behavior of short line items (which was never changed in the docs).

- [feature] [config] Added new argument `Config.config_args`, allows a dictionary of replacement variables to be passed which will serve as substitution values when an API-produced `Config` consumes the `.ini` file. Pull request courtesy Noufal Ibrahim.

  References: pull request bitbucket:33

- [feature] [operations] The `Table` object is now returned when the `Operations.create_table()` method is used. This `Table` is suitable for use in subsequent SQL operations, in particular the `Operations.bulk_insert()` operation.

  References: #205

- [feature] [autogenerate] Indexes and unique constraints are now included in the `EnvironmentContext.configure.include_object` hook. Indexes are sent with type `"index"` and unique constraints with type `"unique_constraint"`.

  References: #203

- [feature] SQLAlchemy’s testing infrastructure is now used to run tests. This system supports both nose and pytest and opens the way for Alembic testing to support any number of backends, parallel testing, and 3rd party dialect testing.

### bug

- [bug] [commands] The `alembic revision` command accepts the `--sql` option to suit some very obscure use case where the `revision_environment` flag is set up, so that `env.py` is run when `alembic revision` is run even though autogenerate isn’t specified. As this flag is otherwise confusing, error messages are now raised if `alembic revision` is invoked with both `--sql` and `--autogenerate` or with `--sql` without `revision_environment` being set.

  References: #248

- [bug] [postgresql] [autogenerate] Added a rule for Postgresql to not render a “drop unique” and “drop index” given the same name; for now it is assumed that the “index” is the implicit one Postgreql generates. Future integration with new SQLAlchemy 1.0 features will improve this to be more resilient.

  References: #247

- [bug] [autogenerate] A change in the ordering when columns and constraints are dropped; autogenerate will now place the “drop constraint” calls before the “drop column” calls, so that columns involved in those constraints still exist when the constraint is dropped.

  References: #247

- [bug] [oracle] The Oracle dialect sets “transactional DDL” to False by default, as Oracle does not support transactional DDL.

  References: #245

- [bug] [autogenerate] Fixed a variety of issues surrounding rendering of Python code that contains unicode literals. The first is that the “quoted_name” construct that SQLAlchemy uses to represent table and column names as well as schema names does not `repr()` correctly on Py2K when the value contains unicode characters; therefore an explicit stringification is added to these. Additionally, SQL expressions such as server defaults were not being generated in a unicode-safe fashion leading to decode errors if server defaults contained non-ascii characters.

  References: #243

- [bug] [operations] The `Operations.add_column()` directive will now additionally emit the appropriate `CREATE INDEX` statement if the `Column` object specifies `index=True`. Pull request courtesy David Szotten.

  References: #174, pull request bitbucket:29

- [bug] [autogenerate] Bound parameters are now resolved as “literal” values within the SQL expression inside of a CheckConstraint(), when rendering the SQL as a text string; supported for SQLAlchemy 0.8.0 and forward.

  References: #219

- [bug] [autogenerate] Added a workaround for SQLAlchemy issue #3023 (fixed in 0.9.5) where a column that’s part of an explicit PrimaryKeyConstraint would not have its “nullable” flag set to False, thus producing a false autogenerate. Also added a related correction to MySQL which will correct for MySQL’s implicit server default of ‘0’ when a NULL integer column is turned into a primary key column.

  References: #199

- [bug] [autogenerate] [mysql] Repaired issue related to the fix for #208 and others; a composite foreign key reported by MySQL would cause a KeyError as Alembic attempted to remove MySQL’s implicitly generated indexes from the autogenerate list.

  References: #240

- [bug] [autogenerate] If the “alembic_version” table is present in the target metadata, autogenerate will skip this also. Pull request courtesy Dj Gilcrease.

  References: #28

- [bug] [autogenerate] The `EnvironmentContext.configure.version_table` and `EnvironmentContext.configure.version_table_schema` arguments are now honored during the autogenerate process, such that these names will be used as the “skip” names on both the database reflection and target metadata sides.

  References: #77

- [bug] [templates] Revision files are now written out using the `'wb'` modifier to `open()`, since Mako reads the templates with `'rb'`, thus preventing CRs from being doubled up as has been observed on windows. The encoding of the output now defaults to ‘utf-8’, which can be configured using a newly added config file parameter `output_encoding`.

  References: #234

- [bug] [operations] Added support for use of the `quoted_name` construct when using the `schema` argument within operations. This allows a name containing a dot to be fully quoted, as well as to provide configurable quoting on a per-name basis.

  References: #230

- [bug] [postgresql] [autogenerate] Added a routine by which the Postgresql Alembic dialect inspects the server default of INTEGER/BIGINT columns as they are reflected during autogenerate for the pattern `nextval(<name>...)` containing a potential sequence name, then queries `pg_catalog` to see if this sequence is “owned” by the column being reflected; if so, it assumes this is a SERIAL or BIGSERIAL column and the server default is omitted from the column reflection as well as any kind of server_default comparison or rendering, along with an INFO message in the logs indicating this has taken place. This allows SERIAL/BIGSERIAL columns to keep the SEQUENCE from being unnecessarily present within the autogenerate operation.

  References: #73

- [bug] [autogenerate] The system by which autogenerate renders expressions within a `Index`, the `server_default` of `Column`, and the `existing_server_default` of `Operations.alter_column()` has been overhauled to anticipate arbitrary SQLAlchemy SQL constructs, such as `func.somefunction()`, `cast()`, `desc()`, and others. The system does not, as might be preferred, render the full-blown Python expression as originally created within the application’s source code, as this would be exceedingly complex and difficult. Instead, it renders the SQL expression against the target backend that’s subject to the autogenerate, and then renders that SQL inside of a `text()` construct as a literal SQL string. This approach still has the downside that the rendered SQL construct may not be backend-agnostic in all cases, so there is still a need for manual intervention in that small number of cases, but overall the majority of cases should work correctly now. Big thanks to Carlos Rivera for pull requests and support on this.

  References: #197, #196, #64

- [bug] [operations] The “match” keyword is not sent to `ForeignKeyConstraint` by `Operations.create_foreign_key()` when SQLAlchemy 0.7 is in use; this keyword was added to SQLAlchemy as of 0.8.0.

## 0.6.7

Released: September 9, 2014

### feature

- [feature] Added support for functional indexes when using the `Operations.create_index()` directive. Within the list of columns, the SQLAlchemy `text()` construct can be sent, embedding a literal SQL expression; the `Operations.create_index()` will perform some hackery behind the scenes to get the `Index` construct to cooperate. This works around some current limitations in `Index` which should be resolved on the SQLAlchemy side at some point.

  References: #222

### bug

- [bug] [mssql] Fixed bug in MSSQL dialect where “rename table” wasn’t using `sp_rename()` as is required on SQL Server. Pull request courtesy Łukasz Bołdys.

  References: pull request bitbucket:26

## 0.6.6

Released: August 7, 2014

### feature

- [feature] Added a new accessor `MigrationContext.config`, when used in conjunction with a `EnvironmentContext` and `Config`, this config will be returned. Patch courtesy Marc Abramowitz.

  References: pull request github:10

### bug

- [bug] A file named `__init__.py` in the `versions/` directory is now ignored by Alembic when the collection of version files is retrieved. Pull request courtesy Michael Floering.

  References: #95, pull request bitbucket:24

- [bug] Fixed Py3K bug where an attempt would be made to sort None against string values when autogenerate would detect tables across multiple schemas, including the default schema. Pull request courtesy paradoxxxzero.

  References: pull request bitbucket:23

- [bug] Autogenerate render will render the arguments within a Table construct using `*[...]` when the number of columns/elements is greater than 255. Pull request courtesy Ryan P. Kelly.

  References: pull request github:15

- [bug] Fixed bug where foreign key constraints would fail to render in autogenerate when a schema name was present. Pull request courtesy Andreas Zeidler.

  References: pull request github:14

- [bug] Some deep-in-the-weeds fixes to try to get “server default” comparison working better across platforms and expressions, in particular on the Postgresql backend, mostly dealing with quoting/not quoting of various expressions at the appropriate time and on a per-backend basis. Repaired and tested support for such defaults as Postgresql interval and array defaults.

  References: #212

- [bug] Liberalized even more the check for MySQL indexes that shouldn’t be counted in autogenerate as “drops”; this time it’s been reported that an implicitly created index might be named the same as a composite foreign key constraint, and not the actual columns, so we now skip those when detected as well.

  References: #208

### misc

- [enhancement] When a run of Alembic command line fails due to `CommandError`, the output now prefixes the string with `"FAILED:"`, and the error is also written to the log output using `log.error()`.

  References: #209

## 0.6.5

Released: May 3, 2014

### feature

- [feature] [environment] Added new feature `EnvironmentContext.configure.transaction_per_migration`, which when True causes the BEGIN/COMMIT pair to incur for each migration individually, rather than for the whole series of migrations. This is to assist with some database directives that need to be within individual transactions, without the need to disable transactional DDL entirely.

  References: #201

### bug

- [bug] [autogenerate] [mysql] This releases’ “autogenerate index detection” bug, when a MySQL table includes an Index with the same name as a column, autogenerate reported it as an “add” even though its not; this is because we ignore reflected indexes of this nature due to MySQL creating them implicitly. Indexes that are named the same as a column are now ignored on MySQL if we see that the backend is reporting that it already exists; this indicates that we can still detect additions of these indexes but not drops, as we cannot distinguish a backend index same-named as the column as one that is user generated or mysql-generated.

  References: #202

- [bug] [autogenerate] Fixed bug where the `include_object()` filter would not receive the original `Column` object when evaluating a database-only column to be dropped; the object would not include the parent `Table` nor other aspects of the column that are important for generating the “downgrade” case where the column is recreated.

  References: #200

- [bug] [environment] Fixed bug where `EnvironmentContext.get_x_argument()` would fail if the `Config` in use didn’t actually originate from a command line call.

  References: #195

- [bug] [autogenerate] Fixed another bug regarding naming conventions, continuing from #183, where add_index() drop_index() directives would not correctly render the `f()` construct when the index contained a convention-driven name.

  References: #194

## 0.6.4

Released: March 28, 2014

### feature

- [feature] The `command.revision()` command now returns the `Script` object corresponding to the newly generated revision. From this structure, one can get the revision id, the module documentation, and everything else, for use in scripts that call upon this command. Pull request courtesy Robbie Coomber.

  References: pull request bitbucket:20

### bug

- [bug] [mssql] Added quoting to the table name when the special EXEC is run to drop any existing server defaults or constraints when the `drop_column.mssql_drop_check` or `drop_column.mssql_drop_default` arguments are used.

  References: #186

- [bug] [mysql] Added/fixed support for MySQL “SET DEFAULT” / “DROP DEFAULT” phrases, which will now be rendered if only the server default is changing or being dropped (e.g. specify None to alter_column() to indicate “DROP DEFAULT”). Also added support for rendering MODIFY rather than CHANGE when the column name isn’t changing.

  References: #103

- [bug] Added support for the `initially`, `match` keyword arguments as well as dialect-specific keyword arguments to `Operations.create_foreign_key()`.

  tags: | feature
  tickets: | 163

  Altered the support for “sourceless” migration files (e.g. only .pyc or .pyo present) so that the flag “sourceless=true” needs to be in alembic.ini for this behavior to take effect.

  References: #190

- [bug] [mssql] The feature that keeps on giving, index/unique constraint autogenerate detection, has even more fixes, this time to accommodate database dialects that both don’t yet report on unique constraints, but the backend does report unique constraints as indexes. The logic Alembic uses to distinguish between “this is an index!” vs. “this is a unique constraint that is also reported as an index!” has now been further enhanced to not produce unwanted migrations when the dialect is observed to not yet implement get_unique_constraints() (e.g. mssql). Note that such a backend will no longer report index drops for unique indexes, as these cannot be distinguished from an unreported unique index.

  References: #185

- [bug] Extensive changes have been made to more fully support SQLAlchemy’s new naming conventions feature. Note that while SQLAlchemy has added this feature as of 0.9.2, some additional fixes in 0.9.4 are needed to resolve some of the issues:

  1. The `Operations` object now takes into account the naming conventions that are present on the `MetaData` object that’s associated using `target_metadata`. When `Operations` renders a constraint directive like `ADD CONSTRAINT`, it now will make use of this naming convention when it produces its own temporary `MetaData` object.
  2. Note however that the autogenerate feature in most cases generates constraints like foreign keys and unique constraints with the final names intact; the only exception are the constraints implicit with a schema-type like Boolean or Enum. In most of these cases, the naming convention feature will not take effect for these constraints and will instead use the given name as is, with one exception....
  3. Naming conventions which use the `"%(constraint_name)s"` token, that is, produce a new name that uses the original name as a component, will still be pulled into the naming convention converter and be converted. The problem arises when autogenerate renders a constraint with it’s already-generated name present in the migration file’s source code, the name will be doubled up at render time due to the combination of #1 and #2. So to work around this, autogenerate now renders these already-tokenized names using the new `Operations.f()` component. This component is only generated if SQLAlchemy 0.9.4 or greater is in use.

  Therefore it is highly recommended that an upgrade to Alembic 0.6.4 be accompanied by an upgrade of SQLAlchemy 0.9.4, if the new naming conventions feature is used.

  See also

  Integration of Naming Conventions into Operations, Autogenerate

  References: #183

- [bug] Suppressed IOErrors which can raise when program output pipe is closed under a program like `head`; however this only works on Python 2. On Python 3, there is not yet a known way to suppress the BrokenPipeError warnings without prematurely terminating the program via signals.

  References: #160

- [bug] Fixed bug where `Operations.bulk_insert()` would not function properly when `Operations.inline_literal()` values were used, either in –sql or non-sql mode. The values will now render directly in –sql mode. For compatibility with “online” mode, a new flag `multiinsert` can be set to False which will cause each parameter set to be compiled and executed with individual INSERT statements.

  References: #179

- [bug] [py3k] Fixed a failure of the system that allows “legacy keyword arguments” to be understood, which arose as of a change in Python 3.4 regarding decorators. A workaround is applied that allows the code to work across Python 3 versions.

  References: #175

## 0.6.3

Released: February 2, 2014

### feature

- [feature] Added new argument `EnvironmentContext.configure.user_module_prefix`. This prefix is applied when autogenerate renders a user-defined type, which here is defined as any type that is from a module outside of the `sqlalchemy.` hierarchy. This prefix defaults to `None`, in which case the `EnvironmentContext.configure.sqlalchemy_module_prefix` is used, thus preserving the current behavior.

  References: #171

- [feature] The `ScriptDirectory` system that loads migration files from a `versions/` directory now supports so-called “sourceless” operation, where the `.py` files are not present and instead `.pyc` or `.pyo` files are directly present where the `.py` files should be. Note that while Python 3.3 has a new system of locating `.pyc`/`.pyo` files within a directory called `__pycache__` (e.g. PEP-3147), PEP-3147 maintains support for the “source-less imports” use case, where the `.pyc`/`.pyo` are in present in the “old” location, e.g. next to the `.py` file; this is the usage that’s supported even when running Python3.3.

  References: #163

### bug

- [bug] Added a workaround for when we call `fcntl.ioctl()` to get at `TERMWIDTH`; if the function returns zero, as is reported to occur in some pseudo-ttys, the message wrapping system is disabled in the same way as if `ioctl()` failed.

  References: #172

- [bug] Added support for autogenerate covering the use case where `Table` objects specified in the metadata have an explicit `schema` attribute whose name matches that of the connection’s default schema (e.g. “public” for Postgresql). Previously, it was assumed that “schema” was `None` when it matched the “default” schema, now the comparison adjusts for this.

  References: #170

- [bug] The `compare_metadata()` public API function now takes into account the settings for `EnvironmentContext.configure.include_object`, `EnvironmentContext.configure.include_symbol`, and `EnvironmentContext.configure.include_schemas`, in the same way that the `--autogenerate` command does. Pull request courtesy Roman Podoliaka.

  References: pull request github:9

- [bug] Calling `bulk_insert()` with an empty list will not emit any commands on the current connection. This was already the case with `--sql` mode, so is now the case with “online” mode.

  References: #168

- [bug] Enabled schema support for index and unique constraint autodetection; previously these were non-functional and could in some cases lead to attribute errors. Pull request courtesy Dimitris Theodorou.

  References: pull request bitbucket:17

- [bug] More fixes to index autodetection; indexes created with expressions like DESC or functional indexes will no longer cause AttributeError exceptions when attempting to compare the columns.

  References: #164

## 0.6.2

Released: Fri Dec 27 2013

### feature

- [feature] [mssql] Added new argument `mssql_drop_foreign_key` to `Operations.drop_column()`. Like `mssql_drop_default` and `mssql_drop_check`, will do an inline lookup for a single foreign key which applies to this column, and drop it. For a column with more than one FK, you’d still need to explicitly use `Operations.drop_constraint()` given the name, even though only MSSQL has this limitation in the first place.

### bug

- [bug] Autogenerate for `op.create_table()` will not include a `PrimaryKeyConstraint()` that has no columns.

- [bug] Fixed bug in the not-internally-used `ScriptDirectory.get_base()` method which would fail if called on an empty versions directory.

- [bug] An almost-rewrite of the new unique constraint/index autogenerate detection, to accommodate a variety of issues. The emphasis is on not generating false positives for those cases where no net change is present, as these errors are the ones that impact all autogenerate runs:

  > - Fixed an issue with unique constraint autogenerate detection where a named `UniqueConstraint` on both sides with column changes would render with the “add” operation before the “drop”, requiring the user to reverse the order manually.
  > - Corrected for MySQL’s apparent addition of an implicit index for a foreign key column, so that it doesn’t show up as “removed”. This required that the index/constraint autogen system query the dialect-specific implementation for special exceptions.
  > - reworked the “dedupe” logic to accommodate MySQL’s bi-directional duplication of unique indexes as unique constraints, and unique constraints as unique indexes. Postgresql’s slightly different logic of duplicating unique constraints into unique indexes continues to be accommodated as well. Note that a unique index or unique constraint removal on a backend that duplicates these may show up as a distinct “remove_constraint()” / “remove_index()” pair, which may need to be corrected in the post-autogenerate if multiple backends are being supported.
  > - added another dialect-specific exception to the SQLite backend when dealing with unnamed unique constraints, as the backend can’t currently report on constraints that were made with this technique, hence they’d come out as “added” on every run.
  > - the `op.create_table()` directive will be auto-generated with the `UniqueConstraint` objects inline, but will not double them up with a separate `create_unique_constraint()` call, which may have been occurring. Indexes still get rendered as distinct `op.create_index()` calls even when the corresponding table was created in the same script.
  > - the inline `UniqueConstraint` within `op.create_table()` includes all the options like `deferrable`, `initially`, etc. Previously these weren’t rendering.

  References: #157

- [bug] [mssql] The MSSQL backend will add the batch separator (e.g. `"GO"`) in `--sql` mode after the final `COMMIT` statement, to ensure that statement is also processed in batch mode. Courtesy Derek Harland.

  References: pull request bitbucket:13

## 0.6.1

Released: Wed Nov 27 2013

### feature

- [feature] Expanded the size of the “slug” generated by “revision” to 40 characters, which is also configurable by new field `truncate_slug_length`; and also split on the word rather than the character; courtesy Frozenball.

  References: pull request bitbucket:12

- [feature] Support for autogeneration detection and rendering of indexes and unique constraints has been added. The logic goes through some effort in order to differentiate between true unique constraints and unique indexes, where there are some quirks on backends like Postgresql. The effort here in producing the feature and tests is courtesy of IJL.

  References: #107

### bug

- [bug] [mysql] Fixed bug where `op.alter_column()` in the MySQL dialect would fail to apply quotes to column names that had mixed casing or spaces.

  References: #152

- [bug] Fixed the output wrapping for Alembic message output, so that we either get the terminal width for “pretty printing” with indentation, or if not we just output the text as is; in any case the text won’t be wrapped too short.

  References: #135

- [bug] Fixes to Py3k in-place compatibity regarding output encoding and related; the use of the new io.* package introduced some incompatibilities on Py2k. These should be resolved, due to the introduction of new adapter types for translating from io.* to Py2k file types, StringIO types. Thanks to Javier Santacruz for help with this.

  References: pull request bitbucket:9

- [bug] Fixed py3k bug where the wrong form of `next()` was being called when using the list_templates command. Courtesy Chris Wilkes.

  References: #145

- [bug] Fixed bug introduced by new `include_object` argument where the inspected column would be misinterpreted when using a user-defined type comparison function, causing a KeyError or similar expression-related error. Fix courtesy Maarten van Schaik.

- [bug] Added the “deferrable” keyword argument to `op.create_foreign_key()` so that `DEFERRABLE` constraint generation is supported; courtesy Pedro Romano.

- [bug] Ensured that strings going to stdout go through an encode/decode phase, so that any non-ASCII characters get to the output stream correctly in both Py2k and Py3k. Also added source encoding detection using Mako’s parse_encoding() routine in Py2k so that the __doc__ of a non-ascii revision file can be treated as unicode in Py2k.

  References: #137

## 0.6.0

Released: Fri July 19 2013

### feature

- [feature] Added new kw argument to `EnvironmentContext.configure()` `include_object`. This is a more flexible version of the `include_symbol` argument which allows filtering of columns as well as tables from the autogenerate process, and in the future will also work for types, constraints and other constructs. The fully constructed schema object is passed, including its name and type as well as a flag indicating if the object is from the local application metadata or is reflected.

  References: #101

- [feature] The output of the `alembic history` command is now expanded to show information about each change on multiple lines, including the full top message, resembling the formatting of git log.

- [feature] Added `alembic.config.Config.cmd_opts` attribute, allows access to the `argparse` options passed to the `alembic` runner.

- [feature] Added new command line argument `-x`, allows extra arguments to be appended to the command line which can be consumed within an `env.py` script by looking at `context.config.cmd_opts.x`, or more simply a new method `EnvironmentContext.get_x_argument()`.

  References: #120

- [feature] Added `-r` argument to `alembic history` command, allows specification of `[start]:[end]` to view a slice of history. Accepts revision numbers, symbols “base”, “head”, a new symbol “current” representing the current migration, as well as relative ranges for one side at a time (i.e. `-r-5:head`, `-rcurrent:+3`). Courtesy Atsushi Odagiri for this feature.

- [feature] Source base is now in-place for Python 2.6 through 3.3, without the need for 2to3. Support for Python 2.5 and below has been dropped. Huge thanks to Hong Minhee for all the effort on this!

  References: #55

### bug

- [bug] Added support for options like “name” etc. to be rendered within CHECK constraints in autogenerate. Courtesy Sok Ann Yap.

  References: #125

- [bug] Repaired autogenerate rendering of ForeignKeyConstraint to include use_alter argument, if present.

### misc

- [misc] Source repository has been moved from Mercurial to Git.

## 0.5.0

Released: Thu Apr 4 2013

Note

Alembic 0.5.0 now requires at least version 0.7.3 of SQLAlchemy to run properly. Support for 0.6 has been dropped.

### feature

- [feature] Added `version_table_schema` argument to `EnvironmentContext.configure()`, complements the `version_table` argument to set an optional remote schema for the version table. Courtesy Christian Blume.

  References: #76

- [feature] Added `output_encoding` option to `EnvironmentContext.configure()`, used with `--sql` mode to apply an encoding to the output stream.

  References: #90

- [feature] Added `Operations.create_primary_key()` operation, will genenerate an ADD CONSTRAINT for a primary key.

  References: #93

- [feature] upgrade and downgrade commands will list the first line of the docstring out next to the version number. Courtesy Hong Minhee.

  References: #115

- [feature] Added –head-only option to “alembic current”, will print current version plus the symbol “(head)” if this version is the head or not. Courtesy Charles-Axel Dein.

- [feature] The rendering of any construct during autogenerate can be customized, in particular to allow special rendering for user-defined column, constraint subclasses, using new `render_item` argument to `EnvironmentContext.configure()`.

  References: #108

### bug

- [bug] [postgresql] Fixed format of RENAME for table that includes schema with Postgresql; the schema name shouldn’t be in the “TO” field.

  References: #32

- [bug] [mssql] Fixed bug whereby double quoting would be applied to target column name during an `sp_rename` operation.

  References: #109

- [bug] [sqlite] [mysql] transactional_ddl flag for SQLite, MySQL dialects set to False. MySQL doesn’t support it, SQLite does but current pysqlite driver does not.

  References: #112

- [bug] Autogenerate will render additional table keyword arguments like “mysql_engine” and others within op.create_table().

  References: #110

- [bug] Fixed bug whereby create_index() would include in the constraint columns that are added to all Table objects using events, externally to the generation of the constraint. This is the same issue that was fixed for unique constraints in version 0.3.2.

- [bug] Worked around a backwards-incompatible regression in Python3.3 regarding argparse; running “alembic” with no arguments now yields an informative error in py3.3 as with all previous versions. Courtesy Andrey Antukh.

- [bug] A host of argument name changes within migration operations for consistency. Keyword arguments will continue to work on the old name for backwards compatibility, however required positional arguments will not:

  > `Operations.alter_column()` - `name` -> `new_column_name` - old name will work for backwards compatibility.

  > `Operations.create_index()` - `tablename` -> `table_name` - argument is positional.

  > `Operations.drop_index()` - `tablename` -> `table_name` - old name will work for backwards compatibility.

  > `Operations.drop_constraint()` - `tablename` -> `table_name` - argument is positional.

  > `Operations.drop_constraint()` - `type` -> `type_` - old name will work for backwards compatibility

  References: #104

### misc

- [change] SQLAlchemy 0.6 is no longer supported by Alembic - minimum version is 0.7.3, full support is as of 0.7.9.

## 0.4.2

Released: Fri Jan 11 2013

### feature

- [feature] Added a README.unittests with instructions for running the test suite fully.

  References: #96

### bug

- [bug] [autogenerate] Fixed bug where autogenerate would fail if a Column to be added to a table made use of the ”.key” paramter.

  References: #99

- [bug] [sqlite] The “implicit” constraint generated by a type such as Boolean or Enum will not generate an ALTER statement when run on SQlite, which does not support ALTER for the purpose of adding/removing constraints separate from the column def itself. While SQLite supports adding a CHECK constraint at the column level, SQLAlchemy would need modification to support this. A warning is emitted indicating this constraint cannot be added in this scenario.

  References: #98

- [bug] Added a workaround to setup.py to prevent “NoneType” error from occuring when “setup.py test” is run.

  References: #96

- [bug] Added an append_constraint() step to each condition within test_autogenerate:AutogenRenderTest.test_render_fk_constraint_kwarg if the SQLAlchemy version is less than 0.8, as ForeignKeyConstraint does not auto-append prior to 0.8.

  References: #96

## 0.4.1

Released: Sun Dec 9 2012

### feature

- [feature] Explicit error message describing the case when downgrade –sql is used without specifying specific start/end versions.

  References: #66

### bug

- [bug] Added support for autogenerate render of ForeignKeyConstraint options onupdate, ondelete, initially, and deferred.

  References: #92

- [bug] Autogenerate will include “autoincrement=False” in the rendered table metadata if this flag was set to false on the source `Column` object.

  References: #94

- [bug] Removed erroneous “emit_events” attribute from operations.create_table() documentation.

  References: #81

- [bug] Fixed the minute component in file_template which returned the month part of the create date.

## 0.4.0

Released: Mon Oct 01 2012

### feature

- [feature] Support for tables in alternate schemas has been added fully to all operations, as well as to the autogenerate feature. When using autogenerate, specifying the flag include_schemas=True to Environment.configure() will also cause autogenerate to scan all schemas located by Inspector.get_schema_names(), which is supported by some (but not all) SQLAlchemy dialects including Postgresql. Enormous thanks to Bruno Binet for a huge effort in implementing as well as writing tests. .

  References: #33

- [feature] The command line runner has been organized into a reusable CommandLine object, so that other front-ends can re-use the argument parsing built in.

  References: #70

- [feature] Added “stdout” option to Config, provides control over where the “print” output of commands like “history”, “init”, “current” etc. are sent.

  References: #43

- [feature] Added support for alteration of MySQL columns that have AUTO_INCREMENT, as well as enabling this flag. Courtesy Moriyoshi Koizumi.

### bug

- [bug] Fixed the “multidb” template which was badly out of date. It now generates revision files using the configuration to determine the different upgrade_<xyz>() methods needed as well, instead of needing to hardcode these. Huge thanks to BryceLohr for doing the heavy lifting here.

  References: #71

- [bug] Fixed the regexp that was checking for .py files in the version directory to allow any .py file through. Previously it was doing some kind of defensive checking, probably from some early notions of how this directory works, that was prohibiting various filename patterns such as those which begin with numbers.

  References: #72

- [bug] Fixed MySQL rendering for server_default which didn’t work if the server_default was a generated SQL expression. Courtesy Moriyoshi Koizumi.

## 0.3.6

Released: Wed Aug 15 2012

### feature

- [feature] Added include_symbol option to EnvironmentContext.configure(), specifies a callable which will include/exclude tables in their entirety from the autogeneration process based on name.

  References: #27

- [feature] Added year, month, day, hour, minute, second variables to file_template.

  References: #59

- [feature] Added ‘primary’ to the list of constraint types recognized for MySQL drop_constraint().

- [feature] Added –sql argument to the “revision” command, for the use case where the “revision_environment” config option is being used but SQL access isn’t desired.

### bug

- [bug] Repaired create_foreign_key() for self-referential foreign keys, which weren’t working at all.

- [bug] ‘alembic’ command reports an informative error message when the configuration is missing the ‘script_directory’ key.

  References: #63

- [bug] Fixes made to the constraints created/dropped alongside so-called “schema” types such as Boolean and Enum. The create/drop constraint logic does not kick in when using a dialect that doesn’t use constraints for these types, such as postgresql, even when existing_type is specified to alter_column(). Additionally, the constraints are not affected if existing_type is passed but type_ is not, i.e. there’s no net change in type.

  References: #62

- [bug] Improved error message when specifiying non-ordered revision identifiers to cover the case when the “higher” rev is None, improved message overall.

  References: #66

## 0.3.5

Released: Sun Jul 08 2012

### feature

- [feature] Implemented SQL rendering for CheckConstraint() within autogenerate upgrade, including for literal SQL as well as SQL Expression Language expressions.

### bug

- [bug] Fixed issue whereby reflected server defaults wouldn’t be quoted correctly; uses repr() now.

  References: #31

- [bug] Fixed issue whereby when autogenerate would render create_table() on the upgrade side for a table that has a Boolean type, an unnecessary CheckConstraint() would be generated.

  References: #58

## 0.3.4

Released: Sat Jun 02 2012

### bug

- [bug] Fixed command-line bug introduced by the “revision_environment” feature.

## 0.3.3

Released: Sat Jun 02 2012

### feature

- [feature] New config argument “revision_environment=true”, causes env.py to be run unconditionally when the “revision” command is run, to support script.py.mako templates with dependencies on custom “template_args”.

- [feature] Added “template_args” option to configure() so that an env.py can add additional arguments to the template context when running the “revision” command. This requires either –autogenerate or the configuration directive “revision_environment=true”.

- [feature] Added version_table argument to EnvironmentContext.configure(), allowing for the configuration of the version table name.

  References: #34

- [feature] Added support for “relative” migration identifiers, i.e. “alembic upgrade +2”, “alembic downgrade -1”. Courtesy Atsushi Odagiri for this feature.

### bug

- [bug] Added “type” argument to op.drop_constraint(), and implemented full constraint drop support for MySQL. CHECK and undefined raise an error. MySQL needs the constraint type in order to emit a DROP CONSTRAINT.

  References: #44

- [bug] Fixed bug whereby directories inside of the template directories, such as __pycache__ on Pypy, would mistakenly be interpreted as files which are part of the template.

  References: #49

## 0.3.2

Released: Mon Apr 30 2012

### feature

- [feature] Basic support for Oracle added, courtesy shgoh.

  References: #40

- [feature] Added support for UniqueConstraint in autogenerate, courtesy Atsushi Odagiri

### bug

- [bug] Fixed support of schema-qualified ForeignKey target in column alter operations, courtesy Alexander Kolov.

- [bug] Fixed bug whereby create_unique_constraint() would include in the constraint columns that are added to all Table objects using events, externally to the generation of the constraint.

## 0.3.1

Released: Sat Apr 07 2012

### bug

- [bug] bulk_insert() fixes:

  > 1. bulk_insert() operation was not working most likely since the 0.2 series when used with an engine.
  > 2. Repaired bulk_insert() to complete when used against a lower-case-t table and executing with only one set of parameters, working around SQLAlchemy bug #2461 in this regard.
  > 3. bulk_insert() uses “inline=True” so that phrases like RETURNING and such don’t get invoked for single-row bulk inserts.
  > 4. bulk_insert() will check that you’re passing a list of dictionaries in, raises TypeError if not detected.

  References: #41

## 0.3.0

Released: Thu Apr 05 2012

### feature

- [feature] Added a bit of autogenerate to the public API in the form of the function alembic.autogenerate.compare_metadata.

### misc

- [general] The focus of 0.3 is to clean up and more fully document the public API of Alembic, including better accessors on the MigrationContext and ScriptDirectory objects. Methods that are not considered to be public on these objects have been underscored, and methods which should be public have been cleaned up and documented, including:

  > MigrationContext.get_current_revision() ScriptDirectory.iterate_revisions() ScriptDirectory.get_current_head() ScriptDirectory.get_heads() ScriptDirectory.get_base() ScriptDirectory.generate_revision()

## 0.2.2

Released: Mon Mar 12 2012

### feature

- [feature] Informative error message when op.XYZ directives are invoked at module import time.

- [feature] Added execution_options parameter to op.execute(), will call execution_options() on the Connection before executing.

  The immediate use case here is to allow access to the new no_parameters option in SQLAlchemy 0.7.6, which allows some DBAPIs (psycopg2, MySQLdb) to allow percent signs straight through without escaping, thus providing cross-compatible operation with DBAPI execution and static script generation.

- [feature] script_location can be interpreted by pkg_resources.resource_filename(), if it is a non-absolute URI that contains colons. This scheme is the same one used by Pyramid.

  References: #29

- [feature] added missing support for onupdate/ondelete flags for ForeignKeyConstraint, courtesy Giacomo Bagnoli

### bug

- [bug] Fixed inappropriate direct call to util.err() and therefore sys.exit() when Config failed to locate the config file within library usage.

  References: #35

- [bug] Autogenerate will emit CREATE TABLE and DROP TABLE directives according to foreign key dependency order.

- [bug] implement ‘tablename’ parameter on drop_index() as this is needed by some backends.

- [bug] setup.py won’t install argparse if on Python 2.7/3.2

- [bug] fixed a regression regarding an autogenerate error message, as well as various glitches in the Pylons sample template. The Pylons sample template requires that you tell it where to get the Engine from now. courtesy Marcin Kuzminski

  References: #30

- [bug] drop_index() ensures a dummy column is added when it calls “Index”, as SQLAlchemy 0.7.6 will warn on index with no column names.

## 0.2.1

Released: Tue Jan 31 2012

### bug

- [bug] Fixed the generation of CHECK constraint, regression from 0.2.0

  References: #26

## 0.2.0

Released: Mon Jan 30 2012

### feature

- [feature] API rearrangement allows everything Alembic does to be represented by contextual objects, including EnvironmentContext, MigrationContext, and Operations. Other libraries and applications can now use things like “alembic.op” without relying upon global configuration variables. The rearrangement was done such that existing migrations should be OK, as long as they use the pattern of “from alembic import context” and “from alembic import op”, as these are now contextual objects, not modules.

  References: #19

- [feature] The naming of revision files can now be customized to be some combination of “rev id” and “slug”, the latter of which is based on the revision message. By default, the pattern “<rev>_<slug>” is used for new files. New script files should include the “revision” variable for this to work, which is part of the newer script.py.mako scripts.

  References: #24

- [feature] Can create alembic.config.Config with no filename, use set_main_option() to add values. Also added set_section_option() which will add sections.

  References: #23

### bug

- [bug] env.py templates call connection.close() to better support programmatic usage of commands; use NullPool in conjunction with create_engine() as well so that no connection resources remain afterwards.

  References: #25

- [bug] fix the config.main() function to honor the arguments passed, remove no longer used “scripts/alembic” as setuptools creates this for us.

  References: #22

- [bug] Fixed alteration of column type on MSSQL to not include the keyword “TYPE”.

## 0.1.1

Released: Wed Jan 04 2012

### feature

- [feature] PyPy is supported.

- [feature] Python 2.5 is supported, needs __future__.with_statement

- [feature] Add alembic_module_prefix argument to configure() to complement sqlalchemy_module_prefix.

  References: #18

### bug

- [bug] Clean up file write operations so that file handles are closed.

- [bug] Fix autogenerate so that “pass” is generated between the two comments if no net migrations were present.

- [bug] Fix autogenerate bug that prevented correct reflection of a foreign-key referenced table in the list of “to remove”.

  References: #16

- [bug] Fix bug where create_table() didn’t handle self-referential foreign key correctly

  References: #17

- [bug] Default prefix for autogenerate directives is “op.”, matching the mako templates.

  References: #18

- [bug] fix quotes not being rendered in ForeignKeConstraint during autogenerate

  References: #14

## 0.1.0

Released: Wed Nov 30 2011

- Initial release. Status of features:

- Alembic is used in at least one production environment, but should still be considered ALPHA LEVEL SOFTWARE as of this release, particularly in that many features are expected to be missing / unimplemented. Major API changes are not anticipated but for the moment nothing should be assumed.

  The author asks that you please report all issues, missing features, workarounds etc. to the bugtracker, at https://bitbucket.org/zzzeek/alembic/issues/new .

- Python 3 is supported and has been tested.

- The “Pylons” and “MultiDB” environment templates have not been directly tested - these should be considered to be samples to be modified as needed. Multiple database support itself is well tested, however.

- Postgresql and MS SQL Server environments have been tested for several weeks in a production environment. In particular, some involved workarounds were implemented to allow fully-automated dropping of default- or constraint-holding columns with SQL Server.

- MySQL support has also been implemented to a basic degree, including MySQL’s awkward style of modifying columns being accommodated.

- Other database environments not included among those three have not been tested, at all. This includes Firebird, Oracle, Sybase. Adding support for these backends should be straightforward. Please report all missing/ incorrect behaviors to the bugtracker! Patches are welcome here but are optional - please just indicate the exact format expected by the target database.

- SQLite, as a backend, has almost no support for schema alterations to existing databases. The author would strongly recommend that SQLite not be used in a migration context - just dump your SQLite database into an intermediary format, then dump it back into a new schema. For dev environments, the dev installer should be building the whole DB from scratch. Or just use Postgresql, which is a much better database for non-trivial schemas. Requests for full ALTER support on SQLite should be reported to SQLite’s bug tracker at http://www.sqlite.org/src/wiki?name=Bug+Reports, as Alembic will not be implementing the “rename the table to a temptable then copy the data into a new table” workaround. Note that Alembic will at some point offer an extensible API so that you can implement commands like this yourself.

- Well-tested directives include add/drop table, add/drop column, including support for SQLAlchemy “schema” types which generate additional CHECK constraints, i.e. Boolean, Enum. Other directives not included here have not been strongly tested in production, i.e. rename table, etc.

- Both “online” and “offline” migrations, the latter being generated SQL scripts to hand off to a DBA, have been strongly production tested against Postgresql and SQL Server.

- Modify column type, default status, nullable, is functional and tested across PG, MSSQL, MySQL, but not yet widely tested in production usage.

- Many migrations are still outright missing, i.e. create/add sequences, etc. As a workaround, execute() can be used for those which are missing, though posting of tickets for new features/missing behaviors is strongly encouraged.

- Autogenerate feature is implemented and has been tested, though only a little bit in a production setting. In particular, detection of type and server default changes are optional and are off by default; they can also be customized by a callable. Both features work but can have surprises particularly the disparity between BIT/TINYINT and boolean, which hasn’t yet been worked around, as well as format changes performed by the database on defaults when it reports back. When enabled, the PG dialect will execute the two defaults to be compared to see if they are equivalent. Other backends may need to do the same thing.

  The autogenerate feature only generates “candidate” commands which must be hand-tailored in any case, so is still a useful feature and is safe to use. Please report missing/broken features of autogenerate! This will be a great feature and will also improve SQLAlchemy’s reflection services.

- Support for non-ASCII table, column and constraint names is mostly nonexistent. This is also a straightforward feature add as SQLAlchemy itself supports unicode identifiers; Alembic itself will likely need fixes to logging, column identification by key, etc. for full support here.

### Navigation

- index
- modules |
- previous |
- Alembic 0.7.0 documentation »